﻿using DbCommon.BusinessCore.BaseCore;
using DbCommon.Enties.DbModels;
using ProjectManagementSystem.Common.Config;
using ProjectManagementSystem.Common.Core;
using ProjectManagementSystem.Common.DataCache;
using ProjectManagementSystem.Common.Extenions;
using ProjectManagementSystem.Common.Function;
using ProjectManagementSystem.Common.Logger;
using ProjectManagementSystem.Common.Models;
using ProjectManagementSystem.Common.Service;
using ProjectManagementSystem.Device.CommandCallback;
using System;
using System.Collections.Concurrent;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading;
using System.Threading.Tasks;

namespace ProjectManagementSystem.Common.Dispatch
{
    public class PackDispatch
    {
        private static PackDispatch m_instance;
        private bool checkOnline = true;
        private bool firstStart = true;
        private bool agvHeartbeat;

        public double TimeConsumption { get; private set; }
        public ConcurrentDictionary<string, PackStationRgv> StationDictionary { get; set; } = new ConcurrentDictionary<string, PackStationRgv>();

        public static PackDispatch Instance
        {
            get
            {
                if (m_instance == null)
                {
                    m_instance = new PackDispatch();
                    if (!m_instance.Initialize())
                    {
                        m_instance = null;
                    }
                }
                return m_instance;
            }
        }


        private PackDispatch()
        {
        }


        private bool Initialize()
        {
            try
            {
                var assemblyName = this.GetType().Assembly.GetName().Name;
                var nameSpace = this.GetType().Namespace;
                string fullClassName = $"{nameSpace}.PackInteractionBase";

                var packPlcToAgv = InstanceConstructor.GetInstance<IPackInteraction>(assemblyName, fullClassName);
                if (packPlcToAgv == null)
                {
                    throw new NullReferenceException("创建的对象为null");
                }


                var configList = ExcelConfig.Instance.RouteConfigList;

                for (int i = 0; i < configList.Count; i++)
                {
                    var config = configList[i];

                    PackStationRgv data = StationDictionary.TryGetValue(config.LocationCode, out var station) ? station : new PackStationRgv();
                    data.Id = config.LocationCode;
                    data.routeConfig = config;

                    var sClassName = config.LocationCode.GetLocationMember("PACK对接类");
                    if (!string.IsNullOrEmpty(sClassName))
                    {
                        var sPackPlcToAgv = InstanceConstructor.GetInstance<IPackInteraction>(assemblyName, $"{nameSpace}.{sClassName}");
                        if (sPackPlcToAgv == null)
                        {
                            throw new NullReferenceException("创建的对象为null");
                        }

                        data.packInteraction = sPackPlcToAgv;
                    }
                    else
                    {
                        data.packInteraction = packPlcToAgv;
                    }


                    data.EnteringRoutes = config.LocationCode.GetLocationMember("AGV正在进入路线").ToValueArray<int>();
                    data.EnteringMarks = config.LocationCode.GetLocationMember("AGV正在进入地标").ToValueArray<int>();

                    data.Mark = config.StationID;
                    data.MarkRequestEnter = config.LocationCode.GetLocationMember("AGV请求进入地标").ToValue<int>();
                    data.MarkLeaveDone = config.LocationCode.GetLocationMember("AGV离开完成地标").ToValue<int>();

                    data.LeavingRoutes = config.LocationCode.GetLocationMember("AGV正在离开路线").ToValueArray<int>();
                    data.LeavingMarks = config.LocationCode.GetLocationMember("AGV正在离开地标").ToValueArray<int>();

                    data.plcEStopAddr = config.LocationCode.GetLocationMember("PLC急停");

                    StationDictionary.AddOrUpdate(data.Id, data, (key, value) => data);
                }

                Task.Factory.StartNew(UserThread, TaskCreationOptions.LongRunning);

                return true;
            }
            catch (Exception ex)
            {
                CLog.Instance.SystemLog.WriteException("PackDispatch Initialize", ex);
            }

            return false;
        }

        private void ReleaseAgvOnStop(PackStationRgv data, InfoAgv agvData, int releaseResult, string posDesc, string releaseMsg)
        {
            if (releaseResult > 0)
            {
                int route = releaseResult < 20000 ? releaseResult : 0;

                DeviceControl.Instance.ReleaseAgvOnStop(agvData.Mark, route, agvData.AgvId);
                CLog.Instance.TaskLog.WriteInfo($"{data.Id} {posDesc} 停止点放行 {agvData.AgvId}# M:{agvData.Mark} R:{route} {releaseMsg}");
            }
        }

        private string GetAgvDataString(InfoAgv agvData)
        {
            return agvData == null ? null : $"({agvData.AgvId}# M:{agvData.Mark} R:{agvData.Route} S:{agvData.StatusEnum})";
        }

        private void UpdateStationDict(bool[] forceWrite)
        {
            var agvDataList1 = DeviceControl.Instance.Communication.InfoAgvDictionary.Values;
            if (agvDataList1 == null) return;

            var agvDataList = checkOnline ? agvDataList1.Where(d => d.DataValid) : agvDataList1.Where(d => true);

            DateTime start = DateTime.Now;
            Parallel.ForEach(StationDictionary.Values, delegate (PackStationRgv data, ParallelLoopState loopState)
            {
                try
                {
                    //心跳信号处理
                    var config = data.routeConfig;
                    if (config.PlcConnected
                        && config.PlcReadWrite != null
                        && !string.IsNullOrEmpty(config.PlcHeartbearAddr_AGV))
                    {
                        var tempResult2 = config.PlcReadWrite.Write(config.PlcHeartbearAddr_AGV, agvHeartbeat);
                        if (!tempResult2.IsSuccess)
                        {
                            CLog.Instance.SystemLog.WriteInfo($"{config.LocationCode} {config.PlcIpAddr} 写入AGV心跳: {tempResult2.IsSuccess.ToChineseString()}  {config.PlcHeartbearAddr_AGV}");
                        }
                    }

                    //计算AGV状态信号
                    data.agvDataAtStation = data.Mark > 0 ? agvDataList.FirstOrDefault(d => d.Mark > 0 && d.Mark == data.Mark && d.StatusEnum == EnumInfoAgvStatus.StopPoint) : null;
                    data.agvDataRequestEnter = data.MarkRequestEnter > 0 ? agvDataList.FirstOrDefault(d => d.Mark > 0 && d.Mark == data.MarkRequestEnter && d.StatusEnum == EnumInfoAgvStatus.StopPoint) : null;
                    data.agvDataLeaveDone = data.MarkLeaveDone > 0 ? agvDataList.FirstOrDefault(d => d.Mark > 0 && d.Mark == data.MarkLeaveDone && d.StatusEnum == EnumInfoAgvStatus.StopPoint) : null;

                    data.agvDataEntering = agvDataList.FirstOrDefault(d => d.Mark > 0 && d.Route > 0 &&
                    (data.EnteringRoutes.Length > 0 ? data.EnteringRoutes.Contains(d.Route) : true) &&
                    ((d.Mark == data.MarkRequestEnter && d.StatusEnum != EnumInfoAgvStatus.StopPoint && d.StatusEnum != EnumInfoAgvStatus.Charging && d.StatusEnum != EnumInfoAgvStatus.Deceleration) ||
                    (data.EnteringMarks.Length > 0 && data.EnteringMarks.Contains(d.Mark))));

                    data.agvDataLeaving = agvDataList.FirstOrDefault(d => d.Mark > 0 && d.Route > 0 &&
                    (data.LeavingRoutes.Length > 0 ? data.LeavingRoutes.Contains(d.Route) : true) &&
                    ((d.Mark == data.Mark && d.StatusEnum != EnumInfoAgvStatus.StopPoint && d.StatusEnum != EnumInfoAgvStatus.Charging && d.StatusEnum != EnumInfoAgvStatus.Deceleration) ||
                    (data.LeavingMarks.Length > 0 && data.LeavingMarks.Contains(d.Mark))));

                    data.AgvInStationInfo = GetAgvDataString(data.agvDataAtStation);
                    data.AgvRequestEnterInfo = GetAgvDataString(data.agvDataRequestEnter);
                    data.AgvLeaveDoneInfo = GetAgvDataString(data.agvDataLeaveDone);
                    data.AgvEnteringInfo = GetAgvDataString(data.agvDataEntering);
                    data.AgvLeavingInfo = GetAgvDataString(data.agvDataLeaving);

                    var result = data.agvDataAtStation != null;
                    if (data.AgvInStation != result || forceWrite[1])
                    {
                        data.AgvInStation = result;
                        data.packInteraction.HandleAgvToPlc(config, "AGV到位", data.AgvInStation, data.AgvInStationInfo);
                    }

                    result = data.agvDataRequestEnter != null;
                    if (data.AgvRequestEnter != result || firstStart || forceWrite[2])
                    {
                        data.AgvRequestEnter = result;
                        data.packInteraction.HandleAgvToPlc(config, "AGV请求进入", data.AgvRequestEnter, data.AgvRequestEnterInfo);
                    }

                    result = data.agvDataLeaveDone != null;
                    if (data.AgvLeaveDone != result || firstStart || forceWrite[3])
                    {
                        data.AgvLeaveDone = result;
                        data.packInteraction.HandleAgvToPlc(config, "AGV离开完成", data.AgvLeaveDone, data.AgvLeaveDoneInfo);
                    }

                    result = data.agvDataEntering != null;
                    if (data.AgvEntering != result || firstStart || forceWrite[4])
                    {
                        data.AgvEntering = result;
                        data.packInteraction.HandleAgvToPlc(config, "AGV正在进入", data.AgvEntering, data.AgvEnteringInfo);
                    }

                    result = data.agvDataLeaving != null;
                    if (data.AgvLeaving != result || firstStart || forceWrite[5])
                    {
                        data.AgvLeaving = result;
                        data.packInteraction.HandleAgvToPlc(config, "AGV正在离开", data.AgvLeaving, data.AgvLeavingInfo);
                    }

                    var agvId = data.agvDataAtStation == null ? 0 : data.agvDataAtStation.AgvId;
                    if (data.AgvId != agvId || forceWrite[1])
                    {
                        data.AgvId = agvId;
                        data.packInteraction.HandleAgvToPlc(config, "AGV编号", (short)data.AgvId, data.AgvInStationInfo);
                    }

                    var alarmId = data.agvDataAtStation?.Alarm ?? 0;
                    if (data.AgvAlarmCode != alarmId || forceWrite[1])
                    {
                        data.AgvAlarmCode = alarmId;
                        data.packInteraction.HandleAgvToPlc(config, "AGV报警代码", (short)data.AgvAlarmCode, data.AgvInStationInfo);
                    }

                    bool agvAlarm = alarmId > 0;
                    if (data.AgvAlarm != agvAlarm || forceWrite[1])
                    {
                        data.AgvAlarm = agvAlarm;
                        data.packInteraction.HandleAgvToPlc(config, "AGV报警", data.AgvAlarm, data.AgvInStationInfo);
                    }


                    //对接停止点放行信号
                    if (data.AgvLeaveDone && data.ReleaseAgvTimeCooled3)
                    {
                        int releaseResult = data.packInteraction.LeaveDone(config, out string releaseMsg);
                        if (data.MessageLeaveDone != releaseMsg)
                        {
                            CLog.Instance.TaskLog.WriteDebug($"{data.Id} AGV离开完成 {releaseMsg}");
                        }
                        data.MessageLeaveDone = releaseMsg;

                        if (releaseResult > 0)
                        {
                            ReleaseAgvOnStop(data, data.agvDataLeaveDone, releaseResult, "AGV离开完成", releaseMsg);
                            data.LastReleaseAgvTime3 = DateTime.Now;
                        }
                    }

                    if (data.AgvInStation && data.ReleaseAgvTimeCooled)
                    {
                        int releaseResult = data.packInteraction.ReleaseAgv(config, out string releaseMsg);
                        if (data.Message != releaseMsg)
                        {
                            CLog.Instance.TaskLog.WriteDebug($"{data.Id} AGV到位 {releaseMsg}");
                        }
                        data.Message = releaseMsg;

                        if (releaseResult > 0)
                        {
                            ReleaseAgvOnStop(data, data.agvDataAtStation, releaseResult, "AGV到位", releaseMsg);
                            data.LastReleaseAgvTime = DateTime.Now;
                        }
                    }

                    if (data.AgvRequestEnter && data.ReleaseAgvTimeCooled2)
                    {
                        int releaseResult = data.packInteraction.AllowEnter(config, out string releaseMsg);
                        if (data.MessageRequestEnter != releaseMsg)
                        {
                            CLog.Instance.TaskLog.WriteDebug($"{data.Id} AGV请求进入 {releaseMsg}");
                        }
                        data.MessageRequestEnter = releaseMsg;

                        if (releaseResult > 0)
                        {
                            ReleaseAgvOnStop(data, data.agvDataRequestEnter, releaseResult, "AGV请求进入", releaseMsg);
                            data.LastReleaseAgvTime2 = DateTime.Now;
                        }
                    }


                    data.Message = data.AgvInStation ? data.Message : null;
                    data.MessageRequestEnter = data.AgvRequestEnter ? data.MessageRequestEnter : null;
                    data.MessageLeaveDone = data.AgvLeaveDone ? data.MessageLeaveDone : null;
                }
                catch (Exception ex)
                {
                    data.Message = ex.Message;
                }

                data.TimeConsumption = Math.Round((DateTime.Now - start).TotalMilliseconds);
            });

            Parallel.ForEach(agvDataList, delegate (InfoAgv agvData, ParallelLoopState loopState)
            {
                try
                {
                    PlcStopAgv(agvData);
                }
                catch (Exception)
                {
                }
            });
        }

        private void PlcStopAgv(InfoAgv agvData)
        {
            //查找AGV所在的正在进入和正在离开的所有工位
            var stations = StationDictionary.Values.Where(d =>
            {
                if (string.IsNullOrEmpty(d.plcEStopAddr)
                || string.IsNullOrEmpty(d.routeConfig.PlcIpAddr))
                {
                    return false;
                }

                return d.agvDataEntering?.AgvId == agvData.AgvId || d.agvDataLeaving?.AgvId == agvData.AgvId;
            });

            if (stations.Count() == 0)
            {
                return;
            }

            string msg = null;
            //从AGV所在的工位中，获取需要停车的工位
            var stopStation = stations.FirstOrDefault(d =>
            {
                int result = d.packInteraction.StopAgv(d.routeConfig, out string tempMsg);
                msg += $"{d.routeConfig.LocationCode} {tempMsg}; ";

                if (result == -1 || result > 0)
                {
                    return true;
                }
                return false;
            });

            bool needPauseAgv = stopStation != null;
            bool agvPaused = agvData.StatusEnum == EnumInfoAgvStatus.Pause;

            if (needPauseAgv)
            {
                if (!agvPaused)
                {
                    DeviceControl.Instance.RailAgv_StopButton(agvData.AgvId);
                    CLog.Instance.TaskLog.WriteInfo($"{stopStation?.Id} {agvData.AgvId}# 暂停小车 {msg}");
                }
            }
            else
            {
                if (agvPaused)
                {
                    DeviceControl.Instance.RailAgv_StartButton(agvData.AgvId);
                    CLog.Instance.TaskLog.WriteInfo($"{stopStation?.Id} {agvData.AgvId}# 启动小车 {msg}");
                }
            }
        }

        private void UserThread()
        {
            CLog.Instance.SystemLog.WriteDebug($"PackDispatch已启动");

            uint couter = 0;
            uint couter2 = 0;
            bool[] forces = new bool[10];
            while (true)
            {
                DateTime loopStartTime = DateTime.Now;

                try
                {
                    //错开刷新
                    var mod = couter2 % 10;
                    if (mod >= 0 && mod <= forces.Length)
                    {
                        for (int i = 0; i < forces.Length; i++)
                        {
                            forces[i] = (mod == i);
                        }
                    }
                    couter2++;

                    //心跳信号取反
                    agvHeartbeat = !agvHeartbeat;

                    DateTime start = DateTime.Now;
                    UpdateStationDict(forces);
                    System.Diagnostics.Trace.WriteLine($"UpdateStationDict {string.Join(" ", forces.Select(d => { return d ? "1" : "0"; }))} 耗时：{(DateTime.Now - start).TotalMilliseconds:F1} ms");
                }
                catch (Exception ex)
                {
                    CLog.Instance.SystemLog.WriteException("PackDispatch", ex);
                    Thread.Sleep(5000);
                }

                firstStart = false;
                couter++;

                TimeConsumption = (DateTime.Now - loopStartTime).TotalMilliseconds;
                Thread.Sleep(500);
            }
        }
    }

    public class PackStationRgv
    {
        internal string plcEStopAddr;

        internal int[] EnteringMarks;
        internal int[] EnteringRoutes;
        internal int[] LeavingMarks;
        internal int[] LeavingRoutes;

        internal InfoAgv agvDataAtStation;
        internal InfoAgv agvDataRequestEnter;
        internal InfoAgv agvDataLeaveDone;
        internal InfoAgv agvDataEntering;
        internal InfoAgv agvDataLeaving;

        internal RouteConfig routeConfig { get; set; }
        internal IPackInteraction packInteraction { get; set; }

        public string Id { get; set; }
        public int Mark { get; set; }
        public int MarkRequestEnter { get; set; }
        public int MarkLeaveDone { get; set; }

        public bool AgvRequestEnter { get; set; }
        public bool AgvEntering { get; set; }
        public bool AgvInStation { get; set; }
        public bool AgvLeaving { get; set; }
        public bool AgvLeaveDone { get; set; }
        public bool AgvInArea { get { return AgvEntering || AgvInStation || AgvLeaving; } }

        public bool AgvAlarm { get; set; }
        public int AgvId { get; set; }
        public int AgvAlarmCode { get; set; }

        public string AgvRequestEnterInfo { get; set; }
        public string AgvEnteringInfo { get; set; }
        public string AgvInStationInfo { get; set; }
        public string AgvLeavingInfo { get; set; }
        public string AgvLeaveDoneInfo { get; set; }

        public double TimeConsumption { get; set; }

        public string Message { get; set; }
        public string MessageRequestEnter { get; set; }
        public string MessageLeaveDone { get; set; }

        internal DateTime LastReleaseAgvTime { get; set; }
        internal DateTime LastReleaseAgvTime2 { get; set; }
        internal DateTime LastReleaseAgvTime3 { get; set; }
        internal bool ReleaseAgvTimeCooled { get { return (DateTime.Now - LastReleaseAgvTime).TotalSeconds > 1; } }
        internal bool ReleaseAgvTimeCooled2 { get { return (DateTime.Now - LastReleaseAgvTime2).TotalSeconds > 1; } }
        internal bool ReleaseAgvTimeCooled3 { get { return (DateTime.Now - LastReleaseAgvTime3).TotalSeconds > 1; } }
    }

    public interface IPackInteraction
    {
        int AllowEnter(RouteConfig config, out string msg);
        int ReleaseAgv(RouteConfig config, out string msg);
        int LeaveDone(RouteConfig config, out string msg);
        int StopAgv(RouteConfig config, out string msg);

        /// <summary>
        /// 将工位AGV相关的数据给线体
        /// </summary>
        /// <typeparam name="T">泛型</typeparam>
        /// <param name="config">工位</param>
        /// <param name="plcAddrDesc">地址描述</param>
        /// <param name="agvValue">数据值</param>
        /// <param name="agvInfo">AGV信息</param>
        void HandleAgvToPlc<T>(RouteConfig config, string plcAddrDesc, T agvValue, string agvInfo);
    }

    public class PackInteractionBase : IPackInteraction
    {
        protected int successCode = 20000;
        protected int unnecessaryCode = -20000;
        protected int failedCode = -1;

        protected int ReadPlcValue(RouteConfig config, string plcAddrDesc, out string msg)
        {
            if (string.IsNullOrEmpty(config.PlcIpAddr))
            {
                msg = $"没有配置IP地址(PLC)";
                return unnecessaryCode;
            }

            var plcAddr = config.LocationCode.GetLocationMember(plcAddrDesc);
            if (string.IsNullOrEmpty(plcAddr))
            {
                msg = $"没有配置地址：{plcAddrDesc}";
                return unnecessaryCode;
            }

            if (!config.PlcConnected)
            {
                msg = $"PLC未连接";
                return failedCode;
            }

            var read = config.PlcReadWrite.ReadBool(plcAddr);
            if (!read.IsSuccess)
            {
                msg = $"读取{plcAddrDesc}{plcAddr}失败 Code:{read.ErrorCode} Message:{read.Message}";
                return failedCode;
            }

            msg = $"读取{plcAddrDesc}{plcAddr} = {read.Content}";
            return read.Content ? successCode : 0;
        }

        public virtual int AllowEnter(RouteConfig config, out string msg)
        {
            return ReadPlcValue(config, "PLC允许进入", out msg);
        }

        public virtual int ReleaseAgv(RouteConfig config, out string msg)
        {
            return ReadPlcValue(config, "PLC放行", out msg);
        }

        public virtual int LeaveDone(RouteConfig config, out string msg)
        {
            return ReadPlcValue(config, "PLC离开完成", out msg);
        }

        public virtual int StopAgv(RouteConfig config, out string msg)
        {
            return ReadPlcValue(config, "PLC急停", out msg);
        }


        public virtual void HandleAgvToPlc<T>(RouteConfig config, string plcAddrDesc, T agvValue, string agvInfo)
        {
            if (!config.PlcConnected || config.PlcReadWrite == null) return;

            string plcAddr = config.LocationCode.GetLocationMember(plcAddrDesc);
            if (string.IsNullOrWhiteSpace(plcAddr)) return;

            var readResult = config.PlcReadWrite.ReadValue(plcAddr, agvValue, out var _);
            if (readResult) return;

            var result = config.PlcReadWrite.WriteValue(plcAddr, agvValue);
            CLog.Instance.TaskLog.WriteInfo($"{config.LocationCode} {config.PlcIpAddr} {plcAddrDesc}: {plcAddr} = {agvValue} 写入{result.IsSuccess.ToChineseString()} {agvInfo}");
        }
    }

    public class PackInteractionReleaseReverse : PackInteractionBase
    {
        public override int ReleaseAgv(RouteConfig config, out string msg)
        {
            if (string.IsNullOrEmpty(config.PlcIpAddr))
            {
                msg = $"没有配置IP地址(PLC)";
                return unnecessaryCode;
            }

            var plcReleaseAddr = config.LocationCode.GetLocationMember("PLC放行");
            if (string.IsNullOrEmpty(plcReleaseAddr))
            {
                msg = $"没有配置地址：PLC放行";
                return unnecessaryCode;
            }

            if (!config.PlcConnected)
            {
                msg = $"PLC未连接";
                return failedCode;
            }

            var read = config.PlcReadWrite.ReadBool(plcReleaseAddr);
            if (!read.IsSuccess)
            {
                msg = $"读取{plcReleaseAddr}失败";
                return failedCode;
            }

            msg = $"读取{plcReleaseAddr} = {read.Content}";

            return !read.Content ? successCode : 0;
        }
    }

    public class PackInteractionLineTPA : PackInteractionBase
    {
        public override int AllowEnter(RouteConfig config, out string msg)
        {
            msg = "无需处理";
            return failedCode;
        }
        public override int LeaveDone(RouteConfig config, out string msg)
        {
            if (string.IsNullOrEmpty(config.PlcIpAddr))
            {
                msg = $"没有配置IP地址(PLC)";
                return unnecessaryCode;
            }

            var plcReleaseAddr = config.LocationCode.GetLocationMember("PLC放行");
            if (string.IsNullOrEmpty(plcReleaseAddr))
            {
                msg = $"没有配置地址：PLC放行";
                return unnecessaryCode;
            }

            if (!config.PlcConnected)
            {
                msg = $"PLC未连接";
                return failedCode;
            }

            var read = config.PlcReadWrite.ReadBool(plcReleaseAddr);
            if (!read.IsSuccess)
            {
                msg = $"读取{plcReleaseAddr}失败";
                return failedCode;
            }

            msg = $"读取{plcReleaseAddr} = {read.Content}";

            return !read.Content ? successCode : 0;
        }
    }

    public class PackInteractionMatch : PackInteractionBase
    {
        private bool CheckAllowEnter(string location)
        {
            string plcAllowEnter = location.GetLocationMember("PLC允许进入");
            if (string.IsNullOrEmpty(plcAllowEnter))
            {
                return true;
            }

            var config = ExcelConfig.Instance.GetRouteConfig(location);
            if (config == null
            || config.PlcReadWrite == null
            || config.PlcConnected == false)
            {
                return false;
            }

            var read = config.PlcReadWrite.ReadBool(plcAllowEnter);
            return read.IsSuccess && read.Content;
        }

        private bool CheckNoAgv(string location)
        {
            var ignoreCheck = location.GetLocationMember("忽略检查工位AGV").ToValue<bool>();
            if (ignoreCheck)
            {
                return true;
            }

            var sDict = PackDispatch.Instance.StationDictionary;
            return sDict.TryGetValue(location, out var station) && !station.AgvInArea;
        }

        public override int ReleaseAgv(RouteConfig config, out string msg)
        {
            string allNextLocation = config.LocationCode.GetLocationMember("下个工位");

            string[] nextLocations = allNextLocation.ToValueArray<string>();
            bool[] sAllowArray = new bool[nextLocations.Length];
            bool[] sNoAgvArray = new bool[nextLocations.Length];

            StringBuilder stringBuilder = new StringBuilder();
            for (int i = 0; i < nextLocations.Length; i++)
            {
                var location = nextLocations[i];
                sAllowArray[i] = CheckAllowEnter(location);
                sNoAgvArray[i] = CheckNoAgv(location);

                stringBuilder.Append($"{location} 允许进入{sAllowArray[i]} 无小车{sNoAgvArray[i]}; ");
            }

            string nextLocation = null;
            for (int i = 0; i < nextLocations.Length; i++)
            {
                var location = nextLocations[i];

                if (sAllowArray[i] && sNoAgvArray[i])
                {
                    nextLocation = location;
                    break;
                }
            }

            if (string.IsNullOrEmpty(nextLocation))
            {
                msg = $"{allNextLocation} 未分配 【分配信息{stringBuilder}】";
                return failedCode;
            }
            msg = $"{allNextLocation} 已分配 {nextLocation} 【分配信息{stringBuilder}】";

            var route = nextLocation.GetLocationMember("进工位路线").ToValue<int>();
            if (route <= 0)
            {
                msg = $"获取进工位路线失败 {nextLocation}";
                return failedCode;
            }

            return route;
        }
    }

    public class PackInteractionStart : PackInteractionBase
    {
        private LocationPropertyManager locationManager = new LocationPropertyManager();
        public override int ReleaseAgv(RouteConfig config, out string msg)
        {
            var locationData = locationManager.GetById(config.LocationCode);

            if (locationData == null)
            {
                msg = $"获取库位信息失败 {config.LocationCode}";
                return failedCode;
            }

            msg = $"{config.LocationCode} Status = {locationData.Status}";
            var releaseAgv = locationData.Status == LocationStatus.Filled;

            if (releaseAgv)
            {
                locationData.Status = LocationStatus.Empty;
                locationManager.Update(locationData);
            }

            return releaseAgv ? successCode : 0;
        }
    }

    public class PackInteractionPadRelease : PackInteractionBase
    {
        public override int ReleaseAgv(RouteConfig config, out string msg)
        {
            var dict = CacheStringToObject.ReleaseAgvDtoDict;
            if (!dict.TryGetValue(config.LocationCode, out var releaseAgvData))
            {
                msg = $"获取放行数据失败 {config.LocationCode}";
                return failedCode;
            }

            if (!releaseAgvData.DataValid)
            {
                msg = $"获取放行数据超时 {config.LocationCode}";
                return failedCode;
            }

            releaseAgvData.Status = 1;
            msg = $"获取放行数据成功 {config.LocationCode}";
            return successCode;
        }
    }

    public class PackInteractionCallBox : PackInteractionBase
    {
        public override int ReleaseAgv(RouteConfig config, out string msg)
        {
            int[] marks = config.LocationCode.GetLocationMember("放行叫料路线").ToValueArray<int>();
            if (marks == null || marks.Length == 0)
            {
                msg = "未配置放行叫料路线";
                return unnecessaryCode;
            }

            int count = DeviceControl.Instance.Communication.InfoBoxDictionary.Values
                .Count(d => d.DataValid && marks.Contains(d.Route) && d.BoxCalled);

            if (count < marks.Length)
            {
                msg = "等待放行";
                return failedCode;
            }

            for (int j = 0; j < 2; j++)
            {
                for (int i = 0; i < marks.Length; i++)
                {
                    DeviceControl.Instance.CallBoxLightOff(marks[i]);
                    Thread.Sleep(100);
                }
            }

            msg = "CallBox放行";
            return successCode;
        }
    }

    public class PackInteractionReleaseBox : PackInteractionBase
    {
        public override int ReleaseAgv(RouteConfig config, out string msg)
        {
            if (!DeviceControl.Instance.Communication.InfoBoxReleaseDictionary.TryGetValue(config.StationID, out var info))
            {
                msg = "获取ReleaseBox信息失败";
                return failedCode;
            }

            if (!info.DataValid)
            {
                msg = $"获取ReleaseBox已掉线";
                return failedCode;
            }

            if (!info.BoxReleaseAgv)
            {
                msg = "等待ReleaseBox放行";
                return failedCode;
            }

            msg = "ReleaseBox放行";
            return successCode;
        }

        public override void HandleAgvToPlc<T>(RouteConfig config, string plcAddrDesc, T agvValue, string agvInfo)
        {
            switch (plcAddrDesc)
            {
                case "AGV到位":
                    if (agvValue is bool agvInStaiton)
                    {
                        if (agvInStaiton)
                        {
                            DeviceControl.Instance.ReleaseBoxLightOn(config.StationID);
                        }
                        else
                        {
                            DeviceControl.Instance.ReleaseBoxLightOff(config.StationID);
                        }
                    }
                    break;
                default:
                    base.HandleAgvToPlc(config, plcAddrDesc, agvValue, agvInfo);
                    break;
            }
        }
    }

    public class PackInteractionIOBox : PackInteractionBase
    {
        public override int ReleaseAgv(RouteConfig config, out string msg)
        {
            if (config.IOTransID <= 0)
            {
                msg = "获取IO透传盒ID失败";
                return failedCode;
            }

            int id = config.IOTransID;
            if (!DeviceControl.Instance.Communication.InfoIODictionary.TryGetValue(id, out var infoIO))
            {
                msg = $"获取IO透传盒{id}数据失败";
                return failedCode;
            }

            if (!infoIO.DataValid)
            {
                msg = $"获取IO透传盒{id}数据超时";
                return failedCode;
            }

            var addr = config.LocationCode.GetLocationMember("AGV到位");
            if (string.IsNullOrWhiteSpace(addr))
            {
                msg = $"未配置地址“AGV到位”";
                return failedCode;
            }

            var res = DeviceControl.Instance.GetOutputIO(id, addr);
            if (!res.Item1
                || !res.Item2)
            {
                msg = "等待IO透传盒放行";
                return failedCode;
            }

            msg = "IO透传盒放行";
            return successCode;
        }
    }

    public class PackInteractionAreaTraffic : PackInteractionBase
    {
        private string factoryId = AppSetting.TryGetValue<string>("FactoryId");
        public override int ReleaseAgv(RouteConfig config, out string msg)
        {
            if (!PackDispatch.Instance.StationDictionary.TryGetValue(config.LocationCode, out var packStationRgv))
            {
                msg = "获取PACK工位失败";
                return failedCode;
            }

            if (packStationRgv.agvDataAtStation == null)
            {
                msg = "AGV不在工位点";
                return failedCode;
            }

            AreaTrafficDto dto = new AreaTrafficDto();
            dto.Id = config.LocationCode.GetLocationMember("管制区编号");
            dto.FactoryId = factoryId;
            dto.AgvId = packStationRgv.agvDataAtStation.AgvId.ToString();

            var res = PmsCustomApi.Post<object>("AreaTrafficIn", dto);
            msg = res.message;

            return res.code == 20000 ? successCode : 0;
        }

        public override int LeaveDone(RouteConfig config, out string msg)
        {
            if (!PackDispatch.Instance.StationDictionary.TryGetValue(config.LocationCode, out var packStationRgv))
            {
                msg = "获取PACK工位失败";
                return failedCode;
            }

            if (packStationRgv.agvDataLeaveDone == null)
            {
                msg = "AGV不在离开完成点";
                return failedCode;
            }

            AreaTrafficDto dto = new AreaTrafficDto();
            dto.Id = config.LocationCode.GetLocationMember("管制区编号");
            dto.FactoryId = factoryId;
            dto.AgvId = packStationRgv.agvDataLeaveDone.AgvId.ToString();

            var res = PmsCustomApi.Post<object>("AreaTrafficOut", dto);
            msg = res.message;

            return res.code == 20000 ? successCode : 0;
        }
    }

    public class PackInteractionStationTraffic : PackInteractionBase
    {
        public override int ReleaseAgv(RouteConfig config, out string msg)
        {
            if (!PackDispatch.Instance.StationDictionary.TryGetValue(config.LocationCode, out var packStationRgv))
            {
                msg = "获取PACK工位失败";
                return failedCode;
            }

            var agvData = DeviceControl.Instance.Communication.InfoAgvDictionary.Values
                .FirstOrDefault(d => d.DataValid && d.Mark == packStationRgv.Mark && d.StatusEnum != EnumInfoAgvStatus.StopPoint);

            if (agvData != null)
            {
                msg = $"工位地标管制 {agvData.ToSimpleString()}";
                return failedCode;
            }

            agvData = DeviceControl.Instance.Communication.InfoAgvDictionary.Values
                .FirstOrDefault(d => packStationRgv.LeavingMarks.Contains(d.Mark));

            if (agvData != null)
            {
                msg = $"正在离开地标管制 {agvData.ToSimpleString()}";
                return failedCode;
            }

            msg = "工位管制放行";
            return successCode;
        }
    }
}
