﻿using DbCommon.BusinessCore.BaseCore;
using DbCommon.Enties.DbModels;
using ProjectManagementSystem.Common.Config;
using ProjectManagementSystem.Common.Core;
using ProjectManagementSystem.Common.Extenions;
using ProjectManagementSystem.Common.Function;
using ProjectManagementSystem.Common.Logger;
using ProjectManagementSystem.Device.CommandCallback;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading;
using System.Threading.Tasks;

namespace ProjectManagementSystem.Common.Dispatch
{
    /// <summary>
    /// 用于磁导航调度
    /// </summary>
    public class SimpleDispatch
    {
        private CallTaskManager callTaskManager = new CallTaskManager();
        private SimpleStep simpleStep = new SimpleStep();


        private static SimpleDispatch m_instance;
        public static SimpleDispatch Instance
        {
            get
            {
                if (m_instance == null)
                {
                    m_instance = new SimpleDispatch();
                    if (!m_instance.Initialize())
                    {
                        m_instance = null;
                    }
                }
                return m_instance;
            }
        }

        private SimpleDispatch()
        {
        }

        public bool Initialize()
        {
            try
            {
                callTaskManager.Init();
                callTaskManager.DeleteOldData();

                Task.Factory.StartNew(TaskThread, TaskCreationOptions.LongRunning);

                return true;
            }
            catch (Exception ex)
            {
                CLog.Instance.SystemLog.WriteException("SimpleDispatch Initialize", ex);
            }

            return false;
        }

        public void ReleaseAgv(int mark, int route, string taskId = "")
        {
            var agvInfo = DeviceControl.Instance.Communication.InfoAgvDictionary.Values.FirstOrDefault(d => d.DataValid && d.Mark == mark);
            if (agvInfo == null)
            {
                CLog.Instance.TaskLog.WriteInfo($"{taskId} ReleaseAgv Mark{mark} Route{route} No valid AGV data found, ReleaseAgv Cancel!");
                return;
            }

            int agvId = agvInfo.AgvId;

            switch (agvInfo.StatusEnum)
            {
                case EnumInfoAgvStatus.StopPoint:
                    DeviceControl.Instance.ReleaseAgvOnStop(mark, route, agvId);
                    CLog.Instance.TaskLog.WriteInfo($"{taskId} ReleaseAgvOnStop Mark{mark} Route{route} AGV{agvId}");
                    break;
                case EnumInfoAgvStatus.EndPoint:
                    DeviceControl.Instance.ReleaseAgvOnStandby(mark, route, agvId);
                    CLog.Instance.TaskLog.WriteInfo($"{taskId} ReleaseAgvOnStandby Mark{mark} Route{route} AGV{agvId}");
                    break;
                default:
                    CLog.Instance.TaskLog.WriteInfo($"{taskId} ReleaseAgv Mark{mark} Route{route} AGV{agvId} StatusEnum{agvInfo.StatusEnum} ReleaseAgv Cancel!");
                    break;
            }
        }

        private string GetCallTaskString(CallTask taskData)
        {
            return $"{taskData.TaskID} AGV{taskData.Carrier} {taskData.TaskStatus} {taskData.TemplateName} {taskData.AllWarehouse} Current:{taskData.CurrentWarehouse}";
        }

        private void WriteTaskInfo(CallTask taskData, string log)
        {
            if (taskData.Message != log)
            {
                taskData.Message = log;
                callTaskManager.Update(taskData);

                CLog.Instance.TaskLog.WriteInfo($"{GetCallTaskString(taskData)} Message:{taskData.Message}");
            }
        }

        private bool UpdateTaskStatus(CallTask taskData, EnumTaskStatus status)
        {
            switch (status)
            {
                case EnumTaskStatus.叫料:
                    break;
                case EnumTaskStatus.任务创建:
                    taskData.CreateTime = DateTime.Now;
                    break;
                case EnumTaskStatus.任务启动:
                    taskData.StartTime = DateTime.Now;
                    break;
                case EnumTaskStatus.任务完成:
                    taskData.EndTime = DateTime.Now;
                    break;
                default:
                    break;
            }

            taskData.TaskStatus = status;
            bool result = callTaskManager.Update(taskData);
            if (result)
            {
                CLog.Instance.TaskLog.WriteInfo(GetCallTaskString(taskData));
            }
            return result;
        }

        private void CallTaskProc()//叫料任务处理
        {
            List<CallTask> taskDataList = callTaskManager.Query();

            if (taskDataList == null
                || taskDataList.Count == 0)
            {
                return;
            }

            for (int i = 0; i < taskDataList.Count; i++)
            {
                var taskData = taskDataList[i];
                switch (taskData.TaskStatus)
                {
                    case EnumTaskStatus.叫料:
                        UpdateTaskStatus(taskData, EnumTaskStatus.任务创建);
                        break;
                    case EnumTaskStatus.任务创建:
                        {
                            //查找待命点AGV
                            var infoAgv = GetStanByAgv(taskData.CurrentWarehouse, out var msg);
                            if (infoAgv != null)
                            {
                                taskData.Carrier = infoAgv.AgvId;
                                taskData.StepStatus = EnumStepStatus.None;
                                taskData.StepMark = infoAgv.Mark;
                                taskData.StepRoute = 0;
                                taskData.StepLocation = taskData.CurrentWarehouse;

                                //判断任务是否可以启动，可用于上报其他系统信息
                                if (simpleStep.TaskStarting(taskData))
                                {
                                    UpdateTaskStatus(taskData, EnumTaskStatus.任务启动);
                                }
                                else
                                {
                                    WriteTaskInfo(taskData, "任务启动失败");
                                }
                            }
                            else
                            {
                                WriteTaskInfo(taskData, msg);
                            }
                        }
                        break;
                    case EnumTaskStatus.任务手动删除:
                        {
                            UpdateTaskStatus(taskData, EnumTaskStatus.任务手动删除处理);
                        }
                        break;
                    default:
                        break;
                }
            }

        }

        private void StationHasCarProcAllTask()
        {
            var taskList = callTaskManager.GetList(d => d.TaskStatus == EnumTaskStatus.任务启动);
            var infoAgvDict = DeviceControl.Instance.Communication.InfoAgvDictionary;

            for (int i = 0; i < taskList.Count; i++)
            {
                var taskData = taskList[i];
                if (!infoAgvDict.TryGetValue(taskData.Carrier, out var agvInfo))//找到对应AGV的信息
                {
                    WriteTaskInfo(taskData, "没有AGV数据");
                    continue;
                }

                if (!agvInfo.DataValid)
                {
                    WriteTaskInfo(taskData, "AGV已掉线");
                    continue;
                }
                else
                {
                    if (taskData.Message == "AGV已掉线"
                        || taskData.Message == "没有AGV数据")
                    {
                        WriteTaskInfo(taskData, "AGV已在线");
                    }
                }

                if (agvInfo.AgvId != taskData.Carrier)
                {
                    WriteTaskInfo(taskData, "AGV编号不正确");
                    continue;
                }

                switch (taskData.StepStatus)
                {
                    case EnumStepStatus.None:
                        {
                            if (agvInfo.Mark == taskData.StepMark
                                && agvInfo.InStation)
                            {
                                //AGV到指定地标停止点后，确认发车路线
                                if (simpleStep.StepInStation(taskData))
                                {
                                    taskData.StepStatus = EnumStepStatus.Step_Executing;
                                    taskData.StepLocation = taskData.CurrentWarehouse;

                                    callTaskManager.Update(taskData);

                                    WriteTaskInfo(taskData, $"准备发车前往位置{taskData.StepLocation} 地标:{taskData.StepMark} 路线:{taskData.StepRoute}");
                                }
                            }
                            else
                            {
                                WriteTaskInfo(taskData, $"等待到达判断地标...{taskData.StepMark}");
                            }
                        }
                        break;
                    case EnumStepStatus.Step_Executing:
                        {
                            bool arriveStepLocation = false;
                            int[] outRouteArray = taskData.StepLocation.GetLocationMember("出库路线").ToValueArray<int>();
                            int[] outMarkArray = taskData.StepLocation.GetLocationMember("出库地标").ToValueArray<int>();

                            if (outRouteArray.Contains(agvInfo.Route))
                            {
                                WriteTaskInfo(taskData, $"AGV{agvInfo.AgvId}已到{taskData.StepLocation} 出库路线:{agvInfo.Route}");
                                arriveStepLocation = true;
                            }
                            else if (outMarkArray.Contains(agvInfo.Mark))
                            {
                                WriteTaskInfo(taskData, $"AGV{agvInfo.AgvId}已到{taskData.StepLocation} 出库地标:{agvInfo.Mark}");
                                arriveStepLocation = true;
                            }


                            if (arriveStepLocation)
                            {
                                var result = simpleStep.StepArriveLocation(taskData, out string msg);
                                if (!string.IsNullOrWhiteSpace(msg))
                                {
                                    WriteTaskInfo(taskData, msg);
                                }

                                if (result)
                                {
                                    //到达工位时如果不是停止卡状态，直接判断等待到达下个位置

                                    taskData.StepStatus = agvInfo.InStation ? EnumStepStatus.None : EnumStepStatus.Step_Executing;
                                    taskData.StepMark = agvInfo.InStation ? agvInfo.Mark : 0;
                                    taskData.StepRoute = 0;

                                    //已经到达位置
                                    taskData.WarehouseIndex++;
                                    taskData.StepLocation = taskData.CurrentWarehouse;

                                    if (string.IsNullOrEmpty(taskData.CurrentWarehouse))
                                    {
                                        //判断任务是否可以完成，用于上报其他系统信息
                                        if (simpleStep.TaskFinished(taskData))
                                        {
                                            UpdateTaskStatus(taskData, EnumTaskStatus.任务完成);
                                        }
                                    }
                                    else
                                    {
                                        callTaskManager.Update(taskData);
                                    }
                                }

                            }
                            else
                            {
                                if (agvInfo.Mark == taskData.StepMark && agvInfo.InStation)
                                {
                                    WriteTaskInfo(taskData, $"发车前往位置{taskData.StepLocation} 地标:{taskData.StepMark} 路线:{taskData.StepRoute} 当前AGV信息{agvInfo.ToSimpleString()}");
                                    ReleaseAgv(taskData.StepMark, taskData.StepRoute, taskData.TaskID);
                                }
                                else
                                {
                                    WriteTaskInfo(taskData, $"等待到达位置...{taskData.StepLocation} 出库路线:{string.Join(",", outRouteArray)} 出库地标:{string.Join(",", outMarkArray)}");
                                }
                            }
                        }
                        break;
                    default:
                        break;
                }
            }
        }

        private InfoAgv GetStanByAgv(string startLocation, out string msg)
        {
            msg = null;
            //获取已经分配任务的AGV编号集合
            int[] agvIdArray = callTaskManager
                .GetList(d => d.TaskStatus < EnumTaskStatus.任务完成 && d.Carrier > 0)
                .Select(d => d.Carrier)
                .ToArray();

            string standByStr = startLocation.GetLocationMember("待命点地标");
            int[] standByArray = standByStr.ToValueArray<int>();

            //获取停站状态的空闲AGV列表
            var infoList = DeviceControl.Instance.Communication.InfoAgvDictionary.Values
                .Where(d => d.InStation && !agvIdArray.Contains(d.AgvId));

            if (standByArray.Length > 0)
            {
                for (int i = 0; i < standByArray.Length; i++)
                {
                    int mark = standByArray[i];

                    var agvInfo = infoList.FirstOrDefault(d => d.Mark == mark);
                    if (agvInfo != null)
                    {
                        return agvInfo;
                    }
                }

                msg = $"缺待命AGV {startLocation}待命点地标 {standByStr}";
                return null;
            }
            else
            {
                var infoAgv = infoList.OrderBy(d => d.AgvId).FirstOrDefault();
                if (infoAgv == null)
                {
                    msg = $"{startLocation}未配置待命点地标 查找停止点空闲AGV失败";
                }

                return infoAgv;
            }
        }

        private void TaskThread()
        {
            Thread.Sleep(2000);
            CLog.Instance.SystemLog.WriteDebug($"SimpleDispatch已启动");
            byte counter = 0;
            while (true)
            {
                try
                {
                    CallTaskProc();
                    StationHasCarProcAllTask();
                }
                catch (Exception ex)
                {
                    CLog.Instance.TaskLog.WriteException("SimpleDispatch", ex);
                    Thread.Sleep(5000);
                }

                if (counter == byte.MaxValue)
                {
                    callTaskManager.DeleteOldData();
                }

                counter++;
                Thread.Sleep(500);
            }
        }
    }

}
