﻿using HslCommunication;
using HslCommunication.BasicFramework;
using HslCommunication.Core;
using ProjectManagementSystem.Common.Extenions;
using ProjectManagementSystem.Common.Logger;
using ProjectManagementSystem.Device.CommandCallback;
using ProjectManagementSystem.Device.Core;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading;
using System.Threading.Tasks;

namespace ProjectManagementSystem.Common.Profinet
{
    public class HsjmPlcNet2 : IReadWriteNet
    {
        private static readonly object locker = new object();
        private readonly int total;

        private byte[] localData;
        private byte[] remoteData;
        private byte[] readCommand;

        private CommunicationTcpClient client;


        public int ReceiveTimeOut { get; set; } = 1000;
        public DateTime LastUpdateTime { get; set; }
        public bool DataValid
        {
            get { return (DateTime.Now - LastUpdateTime).TotalMilliseconds < ReceiveTimeOut; }
        }

        public HsjmPlcNet2(string ipAddress, int port, int total)
        {
            client = new CommunicationTcpClient(ipAddress, port);
            client.OnLogMsg += LogMsg;
            client.DataReceived += DataReceived;

            this.total = total;
            localData = new byte[total];

            readCommand = new byte[total + 1];
            readCommand[0] = 0x01;
        }

        private void LogMsg(string obj, Exception ex)
        {
            if (ex == null)
            {
                CLog.Instance.DeviceLog.WriteDebug(obj);
            }
            else
            {
                CLog.Instance.DeviceLog.WriteException("HsjmPlcNet2", obj, ex);
            }
        }

        private void DataReceived(string addr, byte[] data)
        {
            if (data[0] == 0x05 && data[1] == 0x06 && data.Length == data[2])
            {
                remoteData = data;
                LastUpdateTime = DateTime.Now;
            }
        }

        public OperateResult<byte[]> ReadFromCoreServer(byte[] send)
        {
            lock (locker)
            {
                if (!client.Online)
                {
                    try
                    {
                        client.Connect();
                    }
                    catch (Exception ex)
                    {
                        return OperateResult.CreateFailedResult<byte[]>(new OperateResult(ex.Message));
                    }
                }

                var sendTime = DateTime.Now;
                client.SendCommand(send);

                while ((DateTime.Now - sendTime).TotalMilliseconds < ReceiveTimeOut)
                {
                    if (LastUpdateTime > sendTime && DataValid)
                    {
                        return OperateResult.CreateSuccessResult(remoteData);
                    }
                    Thread.Sleep(10);
                }

                return OperateResult.CreateFailedResult<byte[]>(new OperateResult($"接收数据超时 {ReceiveTimeOut}ms"));
            }
        }

        public OperateResult<byte[]> Read(string address, ushort length)
        {
            // 获取指令
            if (!int.TryParse(address.Substring(1), out int idx))
            {
                return OperateResult.CreateFailedResult<byte[]>(new OperateResult($"解析地址失败 {address}"));
            }

            //用于检查写入时，直接返回本地数据
            if (address.StartsWith("A"))
            {
                return OperateResult.CreateSuccessResult(localData.Skip(idx).ToArray());
            }

            // 核心交互
            var read = ReadFromCoreServer(readCommand);
            if (!read.IsSuccess)
            {
                if (read.Content == null || read.Content.Length == 0)
                {
                    read.Content = new byte[length];
                }
                return read;
            }

            // 数据解析
            return OperateResult.CreateSuccessResult(SoftBasic.BytesArrayRemoveBegin(read.Content, 3 + idx));
        }

        public OperateResult<T> Read<T>() where T : class, new()
        {
            throw new NotImplementedException();
        }

        public OperateResult<bool[]> ReadBool(string address, ushort length)
        {
            throw new NotImplementedException();
        }

        public OperateResult<bool> ReadBool(string address)
        {
            var read = Read(address, 1);
            if (!read.IsSuccess) return OperateResult.CreateFailedResult<bool>(read);

            // 数据解析
            return OperateResult.CreateSuccessResult(read.Content[0].GetBitVaule(0));
        }

        public OperateResult<T> ReadCustomer<T>(string address) where T : IDataTransfer, new()
        {
            throw new NotImplementedException();
        }

        public OperateResult<double> ReadDouble(string address)
        {
            throw new NotImplementedException();
        }

        public OperateResult<double[]> ReadDouble(string address, ushort length)
        {
            throw new NotImplementedException();
        }

        public OperateResult<float> ReadFloat(string address)
        {
            throw new NotImplementedException();
        }

        public OperateResult<float[]> ReadFloat(string address, ushort length)
        {
            throw new NotImplementedException();
        }

        public OperateResult<short> ReadInt16(string address)
        {
            throw new NotImplementedException();
        }

        public OperateResult<short[]> ReadInt16(string address, ushort length)
        {
            throw new NotImplementedException();
        }

        public OperateResult<int> ReadInt32(string address)
        {
            throw new NotImplementedException();
        }

        public OperateResult<int[]> ReadInt32(string address, ushort length)
        {
            throw new NotImplementedException();
        }

        public OperateResult<long> ReadInt64(string address)
        {
            throw new NotImplementedException();
        }

        public OperateResult<long[]> ReadInt64(string address, ushort length)
        {
            throw new NotImplementedException();
        }

        public OperateResult<string> ReadString(string address, ushort length)
        {
            throw new NotImplementedException();
        }

        public OperateResult<ushort> ReadUInt16(string address)
        {
            throw new NotImplementedException();
        }

        public OperateResult<ushort[]> ReadUInt16(string address, ushort length)
        {
            throw new NotImplementedException();
        }

        public OperateResult<uint> ReadUInt32(string address)
        {
            throw new NotImplementedException();
        }

        public OperateResult<uint[]> ReadUInt32(string address, ushort length)
        {
            throw new NotImplementedException();
        }

        public OperateResult<ulong> ReadUInt64(string address)
        {
            throw new NotImplementedException();
        }

        public OperateResult<ulong[]> ReadUInt64(string address, ushort length)
        {
            throw new NotImplementedException();
        }

        public OperateResult Write(string address, byte[] value)
        {
            // 解析指令
            if (!int.TryParse(address.Substring(1), out int idx))
            {
                return OperateResult.CreateFailedResult<byte[]>(new OperateResult($"解析地址失败 {address}"));
            }

            var writeCommand = new byte[total + 1];
            writeCommand[0] = 0x02;

            lock (locker)
            {
                Array.Copy(value, 0, localData, idx, value.Length);
                Array.Copy(localData, 0, writeCommand, 1, localData.Length);
            }

            // 核心交互
            OperateResult<byte[]> read = ReadFromCoreServer(writeCommand);
            if (!read.IsSuccess) return read;

            // 成功
            return OperateResult.CreateSuccessResult();
        }

        public OperateResult Write(string address, short value)
        {
            throw new NotImplementedException();
        }

        public OperateResult Write(string address, short[] values)
        {
            throw new NotImplementedException();
        }

        public OperateResult Write(string address, ushort value)
        {
            throw new NotImplementedException();
        }

        public OperateResult Write(string address, ushort[] values)
        {
            throw new NotImplementedException();
        }

        public OperateResult Write(string address, int value)
        {
            throw new NotImplementedException();
        }

        public OperateResult Write(string address, int[] values)
        {
            throw new NotImplementedException();
        }

        public OperateResult Write(string address, uint value)
        {
            throw new NotImplementedException();
        }

        public OperateResult Write(string address, uint[] values)
        {
            throw new NotImplementedException();
        }

        public OperateResult Write(string address, long value)
        {
            throw new NotImplementedException();
        }

        public OperateResult Write(string address, long[] values)
        {
            throw new NotImplementedException();
        }

        public OperateResult Write(string address, ulong value)
        {
            throw new NotImplementedException();
        }

        public OperateResult Write(string address, ulong[] values)
        {
            throw new NotImplementedException();
        }

        public OperateResult Write(string address, float value)
        {
            throw new NotImplementedException();
        }

        public OperateResult Write(string address, float[] values)
        {
            throw new NotImplementedException();
        }

        public OperateResult Write(string address, double value)
        {
            throw new NotImplementedException();
        }

        public OperateResult Write(string address, double[] values)
        {
            throw new NotImplementedException();
        }

        public OperateResult Write(string address, string value)
        {
            throw new NotImplementedException();
        }

        public OperateResult Write(string address, string value, int length)
        {
            throw new NotImplementedException();
        }

        public OperateResult Write<T>(T data) where T : class, new()
        {
            throw new NotImplementedException();
        }

        public OperateResult Write(string address, bool[] value)
        {
            throw new NotImplementedException();
        }

        public OperateResult Write(string address, bool value)
        {
            throw new NotImplementedException();
        }

        public OperateResult WriteCustomer<T>(string address, T value) where T : IDataTransfer, new()
        {
            throw new NotImplementedException();
        }
    }
}
