﻿using HslCommunication;
using HslCommunication.Core;
using ProjectManagementSystem.Common.Extenions;
using ProjectManagementSystem.Common.Logger;
using ProjectManagementSystem.Device.Core;
using ProjectManagementSystem.Device.Extenions;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading;
using System.Threading.Tasks;

namespace ProjectManagementSystem.Common.Profinet
{
    /// <summary>
    /// IO透传盒的IReadWriteNet方式实现
    /// </summary>
    public class IOBoxNet : IReadWriteNet
    {
        private static readonly object locker = new object();

        private byte[] remoteData;

        private CommunicationTcpClient client;

        public int ReceiveTimeOut { get; set; } = 2000;
        public DateTime LastUpdateTime { get; set; }
        public bool DataValid
        {
            get { return (DateTime.Now - LastUpdateTime).TotalMilliseconds < ReceiveTimeOut; }
        }

        public IOBoxNet(string ipAddress, int port)
        {
            client = new CommunicationTcpClient(ipAddress, port);
            client.OnLogMsg += LogMsg;
            client.DataReceived += DataReceived;
        }

        private void LogMsg(string obj, Exception ex)
        {
            if (ex == null)
            {
                CLog.Instance.DeviceLog.WriteDebug(obj);
            }
            else
            {
                CLog.Instance.DeviceLog.WriteException(this.GetType().Name.ToString(), obj, ex);
            }
        }

        private void DataReceived(string addr, byte[] data)
        {
            if (data.Length == 14 && data[0] == 0x10 && data[1] == 0x81 && data[data.Length - 1] == 0x03)
            {
                remoteData = data;
                LastUpdateTime = DateTime.Now;
            }
        }

        public OperateResult<byte[]> ReadFromCoreServer(byte[] send)
        {
            lock (locker)
            {
                if (!client.Online)
                {
                    try
                    {
                        client.Connect();
                    }
                    catch (Exception ex)
                    {
                        return OperateResult.CreateFailedResult<byte[]>(new OperateResult(ex.Message));
                    }
                }

                var sendTime = DateTime.Now;
                if (send?.Length > 0)
                {
                    client.SendCommand(send);
                }
                else
                {
                    if (DataValid)
                    {
                        return OperateResult.CreateSuccessResult(remoteData);
                    }
                    else
                    {
                        return OperateResult.CreateFailedResult<byte[]>(new OperateResult($"接收数据超时 {ReceiveTimeOut}ms"));
                    }
                }


                while ((DateTime.Now - sendTime).TotalMilliseconds < ReceiveTimeOut)
                {
                    if (LastUpdateTime > sendTime)
                    {
                        return OperateResult.CreateSuccessResult(remoteData);
                    }
                    Thread.Sleep(20);
                }

                return OperateResult.CreateFailedResult<byte[]>(new OperateResult($"接收数据超时 {ReceiveTimeOut}ms"));
            }
        }


        public OperateResult<byte[]> Read(string address, ushort length)
        {
            throw new NotImplementedException();
        }

        public OperateResult<T> Read<T>() where T : class, new()
        {
            throw new NotImplementedException();
        }

        public OperateResult<bool[]> ReadBool(string address, ushort length)
        {
            throw new NotImplementedException();
        }

        public OperateResult<bool> ReadBool(string address)
        {
            if (!address.StartsWith("X") && !address.StartsWith("Y"))
            {
                return new OperateResult<bool>($"不支持的地址 {address}");
            }

            // 获取指令
            if (!int.TryParse(address.Substring(1), out int idx))
            {
                return new OperateResult<bool>($"解析地址失败 {address}");
            }

            if (idx < 1 || idx > 8)
            {
                return new OperateResult<bool>($"地址偏移只支持1到8 {address}");
            }

            // 核心交互
            var read = ReadFromCoreServer(null);
            if (!read.IsSuccess)
            {
                return OperateResult.CreateFailedResult<bool>(read);
            }

            // 数据解析
            var readValue = address.StartsWith("Y") ?
                read.Content[4].GetBitVaule(idx - 1) :
                read.Content[6].GetBitVaule(idx - 1);

            return OperateResult.CreateSuccessResult(readValue);
        }

        public OperateResult<T> ReadCustomer<T>(string address) where T : IDataTransfer, new()
        {
            throw new NotImplementedException();
        }

        public OperateResult<double> ReadDouble(string address)
        {
            throw new NotImplementedException();
        }

        public OperateResult<double[]> ReadDouble(string address, ushort length)
        {
            throw new NotImplementedException();
        }

        public OperateResult<float> ReadFloat(string address)
        {
            throw new NotImplementedException();
        }

        public OperateResult<float[]> ReadFloat(string address, ushort length)
        {
            throw new NotImplementedException();
        }

        public OperateResult<short> ReadInt16(string address)
        {
            throw new NotImplementedException();
        }

        public OperateResult<short[]> ReadInt16(string address, ushort length)
        {
            throw new NotImplementedException();
        }

        public OperateResult<int> ReadInt32(string address)
        {
            throw new NotImplementedException();
        }

        public OperateResult<int[]> ReadInt32(string address, ushort length)
        {
            throw new NotImplementedException();
        }

        public OperateResult<long> ReadInt64(string address)
        {
            throw new NotImplementedException();
        }

        public OperateResult<long[]> ReadInt64(string address, ushort length)
        {
            throw new NotImplementedException();
        }

        public OperateResult<string> ReadString(string address, ushort length)
        {
            throw new NotImplementedException();
        }

        public OperateResult<ushort> ReadUInt16(string address)
        {
            throw new NotImplementedException();
        }

        public OperateResult<ushort[]> ReadUInt16(string address, ushort length)
        {
            throw new NotImplementedException();
        }

        public OperateResult<uint> ReadUInt32(string address)
        {
            throw new NotImplementedException();
        }

        public OperateResult<uint[]> ReadUInt32(string address, ushort length)
        {
            throw new NotImplementedException();
        }

        public OperateResult<ulong> ReadUInt64(string address)
        {
            throw new NotImplementedException();
        }

        public OperateResult<ulong[]> ReadUInt64(string address, ushort length)
        {
            throw new NotImplementedException();
        }

        public OperateResult Write(string address, byte[] value)
        {
            throw new NotImplementedException();
        }

        public OperateResult Write(string address, short value)
        {
            throw new NotImplementedException();
        }

        public OperateResult Write(string address, short[] values)
        {
            throw new NotImplementedException();
        }

        public OperateResult Write(string address, ushort value)
        {
            throw new NotImplementedException();
        }

        public OperateResult Write(string address, ushort[] values)
        {
            throw new NotImplementedException();
        }

        public OperateResult Write(string address, int value)
        {
            throw new NotImplementedException();
        }

        public OperateResult Write(string address, int[] values)
        {
            throw new NotImplementedException();
        }

        public OperateResult Write(string address, uint value)
        {
            throw new NotImplementedException();
        }

        public OperateResult Write(string address, uint[] values)
        {
            throw new NotImplementedException();
        }

        public OperateResult Write(string address, long value)
        {
            throw new NotImplementedException();
        }

        public OperateResult Write(string address, long[] values)
        {
            throw new NotImplementedException();
        }

        public OperateResult Write(string address, ulong value)
        {
            throw new NotImplementedException();
        }

        public OperateResult Write(string address, ulong[] values)
        {
            throw new NotImplementedException();
        }

        public OperateResult Write(string address, float value)
        {
            throw new NotImplementedException();
        }

        public OperateResult Write(string address, float[] values)
        {
            throw new NotImplementedException();
        }

        public OperateResult Write(string address, double value)
        {
            throw new NotImplementedException();
        }

        public OperateResult Write(string address, double[] values)
        {
            throw new NotImplementedException();
        }

        public OperateResult Write(string address, string value)
        {
            throw new NotImplementedException();
        }

        public OperateResult Write(string address, string value, int length)
        {
            throw new NotImplementedException();
        }

        public OperateResult Write<T>(T data) where T : class, new()
        {
            throw new NotImplementedException();
        }

        public OperateResult Write(string address, bool[] value)
        {
            throw new NotImplementedException();
        }

        public OperateResult Write(string address, bool value)
        {
            if (!address.StartsWith("Y"))
            {
                return new OperateResult($"不支持的地址 {address}");
            }

            if (!int.TryParse(address.Substring(1), out int idx))
            {
                return new OperateResult($"解析地址失败 {address}");
            }

            if (idx < 1 || idx > 8)
            {
                return new OperateResult($"地址偏移只支持1到8 {address}");
            }

            if (remoteData?.Length != 14)
            {
                return new OperateResult($"IO透传板未回复数据");
            }

            int index = idx - 1;
            ushort output = remoteData[4];

            ushort newOutput = value ?
                (ushort)(output | (0x01 << index)) :
                (ushort)(output & (ushort.MaxValue - (0x01 << index)));

            var outputBytes = BitConverter.GetBytes(newOutput);

            var buffer = new byte[14] { 0x10, 0x80, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0x03 };
            buffer[2] = remoteData[2];
            buffer[3] = outputBytes[1];
            buffer[4] = outputBytes[0];
            buffer[12] = buffer.XOR_Check(12);//异或

            // 核心交互
            var read = ReadFromCoreServer(buffer);
            if (!read.IsSuccess)
            {
                return OperateResult.CreateFailedResult<bool>(read);
            }

            // 数据解析
            var readValue = read.Content[4].GetBitVaule(idx - 1);
            if (readValue == value)
            {
                return OperateResult.CreateSuccessResult();
            }
            else
            {
                return new OperateResult("写入失败");
            }
        }

        public OperateResult WriteCustomer<T>(string address, T value) where T : IDataTransfer, new()
        {
            throw new NotImplementedException();
        }
    }
}
