﻿using ProjectManagementSystem.Common.Config;
using ProjectManagementSystem.Common.Logger;
using ProjectManagementSystem.Common.Core;
using ProjectManagementSystem.Common.Extenions;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using ProjectManagementSystem.Common.Service;
using ProjectManagementSystem.Common.Models.Crms;
using HslCommunication;

namespace ProjectManagementSystem.Common.TaskBookEvent
{

    /// <summary>
    /// PLC交互行为码，对接地址动态扩展
    /// </summary>
    /// <typeparam name="T">泛型T</typeparam>
    public class TaskBookBehavior_PLC<T> : BaseTaskBookBehaviorV2
    {
        public PlcBehavior PlcBehavior { get; set; }
        public IPlcBehaviorOperator BehaviorOperator { get; set; }

        public TaskBookBehavior_PLC(int behavior) : base(behavior, null)
        {
            PlcBehavior = ExcelConfig.Instance.PlcBehaviorList.FirstOrDefault(d => d.Behavior == behavior);
            Remark = PlcBehavior.Remark;

            if (!string.IsNullOrWhiteSpace(PlcBehavior.OperatorClassName))
            {
                string assemblyName = this.GetType().Assembly.GetName().Name;
                string nameSpace = this.GetType().Namespace;
                string fullClassName = $"{nameSpace}.{PlcBehavior.OperatorClassName}";
                BehaviorOperator = Common.Function.InstanceConstructor.GetInstance<IPlcBehaviorOperator>(assemblyName, fullClassName);

                if (BehaviorOperator == null)
                {
                    CLog.Instance.SystemLog.WriteDebug($"创建对象失败 {assemblyName} {fullClassName}");
                }
            }

        }


        public override bool TaskBookOperator(TaskData taskData, StepData stepInfo)
        {
            try
            {
                bool result = Operator(taskData, stepInfo);
                if (result)
                {
                    //执行异步类
                    if (!string.IsNullOrEmpty(PlcBehavior.AsyncClassName)
                        && PlcBehavior.AsyncStepOffset > 0)
                    {
                        string assemblyName = this.GetType().Assembly.GetName().Name;
                        string nameSpace = this.GetType().Namespace;
                        string fullClassName = $"{nameSpace}.{PlcBehavior.AsyncClassName}";
                        var instance = Common.Function.InstanceConstructor.GetInstance<AsyncBehavior>(assemblyName, fullClassName);

                        if (instance != null)
                        {
                            instance.Behavior = stepInfo.PmsBehavior;
                            instance.Interval = 100;
                            instance.TaskID = taskData.TaskID;
                            instance.AgvId = taskData.Carrier;
                            instance.MaxStepId = taskData.CurrentStepID + PlcBehavior.AsyncStepOffset;

                            instance.Init();
                            instance.Start();
                        }
                    }
                }

                return result;
            }
            catch (Exception ex)
            {
                LogException(ex);
            }
            return false;
        }

        public bool Operator(TaskData taskData, StepData stepInfo)
        {
            DateTime start = DateTime.Now;

            if (stepInfo.StepID == 1 && string.IsNullOrWhiteSpace(stepInfo.WareHouseID))
            {
                stepInfo.WareHouseID = taskData.StepList
                    .OrderBy(d => d.StepID)
                    .FirstOrDefault(d => !string.IsNullOrWhiteSpace(d.WareHouseID))
                    .WareHouseID;
            }

            //模拟器模式下直接返回成功
            if (PlcBehavior.IsSimulate)
            {
                Log(taskData, stepInfo, $"【模拟器】{GetLog(taskData, stepInfo, null, null)}");
                return true;
            }

            //开始
            DateTime start0 = DateTime.Now;
            string msg1 = null;
            if (BehaviorOperator?.Begin(taskData, stepInfo, out msg1) == false)
            {
                Log(taskData, stepInfo, msg1);
                return false;
            }
            var timeSpan0 = (DateTime.Now - start0).TotalMilliseconds;

            DateTime start1 = DateTime.Now;
            //写入
            if (!Interaction(0, PlcBehavior.DbAddressWrite, PlcBehavior.DbValueWriteStr, taskData, stepInfo))
            {
                return false;
            }
            var timeSpan1 = (DateTime.Now - start1).TotalMilliseconds;

            DateTime start2 = DateTime.Now;
            //读取
            if (!Interaction(1, PlcBehavior.DbAddressRead, PlcBehavior.DbValueReadStr, taskData, stepInfo))
            {
                return false;
            }
            var timeSpan2 = (DateTime.Now - start2).TotalMilliseconds;

            DateTime start3 = DateTime.Now;
            //最终写入
            if (!Interaction(0, PlcBehavior.DbAddressFinalWrite, PlcBehavior.DbValueFinalWriteStr, taskData, stepInfo))
            {
                return false;
            }
            var timeSpan3 = (DateTime.Now - start3).TotalMilliseconds;

            //结束
            DateTime start4 = DateTime.Now;
            string msg2 = null;
            if (BehaviorOperator?.End(taskData, stepInfo, out msg2) == false)
            {
                Log(taskData, stepInfo, msg2);
                return false;
            }
            var timeSpan4 = (DateTime.Now - start4).TotalMilliseconds;

            var totalTimeSpan = (DateTime.Now - start).TotalMilliseconds;

            var log = BehaviorOperator == null ?
                $"{GetLog(taskData, stepInfo, null, null)}【写入{timeSpan1:F1}ms 读取{timeSpan2:F1}ms 最终写入{timeSpan3:F1}ms】总耗时{totalTimeSpan:F1}ms" :
                $"{GetLog(taskData, stepInfo, msg1, msg2)}【开始{timeSpan0:F1}ms 写入{timeSpan1:F1}ms 读取{timeSpan2:F1}ms 最终写入{timeSpan3:F1}ms 结束{timeSpan4:F1}ms】总耗时{totalTimeSpan:F1}ms";

            Log(taskData, stepInfo, log);
            return true;
        }

        protected virtual bool Interaction(int flag, string[] addrArray, string valueArray, TaskData taskData, StepData stepInfo)
        {
            if (addrArray.Length <= 0) return true;

            var routeConfig = ExcelConfig.Instance.GetRouteConfig(stepInfo.WareHouseID);
            if (routeConfig == null)
            {
                Log(taskData, stepInfo, "查找库位配置信息失败");
                return false;
            }
            if (string.IsNullOrEmpty(routeConfig.PlcIpAddr))
            {
                Log(taskData, stepInfo, "PlcIp地址为空");
                return false;
            }

            var plc = routeConfig.PlcReadWrite;

            for (int i = 0; i < addrArray.Length; i++)
            {
                string addrDesc = addrArray[i];
                string plcAddr = stepInfo.WareHouseID.GetPlcTaskAddr(addrDesc);
                if (string.IsNullOrEmpty(plcAddr))
                {
                    Log(taskData, stepInfo, $"查找地址失败：“{addrDesc}”");
                    return false;
                }

                var plcValue = GetArrayValue(valueArray, i);

                if (flag == 0)
                {
                    var result = plc.WriteValue(plcAddr, plcValue);
                    if (!result.IsSuccess)
                    {
                        Log(taskData, stepInfo, $"写入失败({addrDesc}{plcAddr}写入{plcValue}) Code:{result.ErrorCode} {result.Message}");
                        return false;
                    }

                    bool resultRead = plc.ReadValue(plcAddr, plcValue, out OperateResult<T> ret);
                    if (!ret.IsSuccess)
                    {
                        Log(taskData, stepInfo, $"写入检查失败(读取{addrDesc}{plcAddr}失败) Code:{ret.ErrorCode} {ret.Message}");
                        return false;
                    }

                    if (!resultRead)
                    {
                        Log(taskData, stepInfo, $"写入检查失败(读取{addrDesc}{plcAddr} != {plcValue}) 实际值:{ret.Content}");
                        return false;
                    }
                }
                else
                {
                    bool resultRead = plc.ReadValue(plcAddr, plcValue, out OperateResult<T> ret);
                    if (!resultRead)
                    {
                        Log(taskData, stepInfo, $"等待信号({addrDesc}{plcAddr}={plcValue}) {(ret.IsSuccess ? $"实际值:{ret.Content}" : $"Code:{ret.ErrorCode} {ret.Message}")}");
                        return false;
                    }
                }
            }

            return true;
        }

        private string GetLog(TaskData taskData, StepData stepInfo, string beginMsg, string endMsg)
        {
            StringBuilder sb = new StringBuilder();
            sb.Append(PlcBehavior.Remark);

            sb.Append("【");
            if (!string.IsNullOrWhiteSpace(beginMsg))
            {
                sb.Append($" {beginMsg};");
            }

            for (int i = 0; i < PlcBehavior.DbAddressWrite.Length; i++)
            {
                string plcAddr = stepInfo.WareHouseID.GetPlcTaskAddr(PlcBehavior.DbAddressWrite[i]);
                var plcValue = GetArrayValue(PlcBehavior.DbValueWriteStr, i);

                sb.Append($" {plcAddr}已写{plcValue};");
            }

            for (int i = 0; i < PlcBehavior.DbAddressRead.Length; i++)
            {
                string plcAddr = stepInfo.WareHouseID.GetPlcTaskAddr(PlcBehavior.DbAddressRead[i]);
                var plcValue = GetArrayValue(PlcBehavior.DbValueReadStr, i);

                sb.Append($" 已等待{plcAddr}={plcValue};");
            }

            for (int i = 0; i < PlcBehavior.DbAddressFinalWrite.Length; i++)
            {
                string plcAddr = stepInfo.WareHouseID.GetPlcTaskAddr(PlcBehavior.DbAddressFinalWrite[i]);
                var plcValue = GetArrayValue(PlcBehavior.DbValueFinalWriteStr, i);

                sb.Append($" {plcAddr}已最终写{plcValue};");
            }

            if (!string.IsNullOrWhiteSpace(endMsg))
            {
                sb.Append($" {endMsg};");
            }
            sb.Append("】");

            return sb.ToString();
        }

        private T GetArrayValue(string str, int index)
        {
            if (string.IsNullOrEmpty(str)) return default(T);

            var array = str.ToValueArray<T>();
            if (array.Length == 0) return default(T);

            return array.Length > index ? array[index] : array[array.Length - 1];
        }
    }

    public interface IPlcBehaviorOperator
    {
        bool Begin(TaskData taskData, StepData stepData, out string msg);
        bool End(TaskData taskData, StepData stepData, out string msg);
    }

}
