﻿using Prism.Commands;
using ProjectManagementSystem.Common.Config;
using ProjectManagementSystem.Common.Extenions;
using ProjectManagementSystem.Common.Function;
using ProjectManagementSystem.Common.Models;
using ProjectManagementSystem.Common.Service;
using ProjectManagementSystem.Language;
using PropertyChanged;
using System;
using System.Collections.Generic;
using System.Collections.ObjectModel;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows;

namespace ProjectManagementSystemView.ViewModels
{
    [AddINotifyPropertyChangedInterface]
    public class DebugViewModel
    {
        public string P1 { get; set; }
        public string P2 { get; set; }
        public string P3 { get; set; }


        public List<string> WareHouseCollection { get; set; }
        public List<RouteConfig> AllLocationCollection { get; set; }
        public List<RouteConfig> SourceLocationCollection { get; set; }
        public List<RouteConfig> TargetLocationCollection { get; set; }



        [OnChangedMethod(nameof(ReloadSourceWarehouse))]
        public string SourceWarehouse { get; set; }

        [OnChangedMethod(nameof(ReloadTargetWarehouse))]
        public string TargetWarehouse { get; set; }
        public RouteConfig SourceLocation { get; set; }
        public RouteConfig TargetLocation { get; set; }

        private bool testTaskCanExecute = true;
        public bool TestTaskCanExecute
        {
            get { return SourceLocation != null && TargetLocation != null && testTaskCanExecute; }
            set { testTaskCanExecute = value; }
        }


        public DelegateCommand<string> SimpleTaskCommand { get; private set; }
        public DelegateCommand<string> TaskTransportCommand { get; private set; }
        public DelegateCommand<string> AgvCallbackCommand { get; private set; }

        public DebugViewModel()
        {
            AllLocationCollection = ExcelConfig.Instance.RouteConfigList;

            var wareHouseFilters = AppSetting.TryGetValue<string>("WareHouseFilter1").ToValueArray<string>().ToList();
            var wareHouseList = ExcelConfig.Instance.RouteConfigList
                .Where(d => !string.IsNullOrEmpty(d.WareHouseCode))
                .Where(d =>
                {
                    if (wareHouseFilters.Count == 0) return true;
                    return wareHouseFilters.Contains(d.WareHouseCode);
                })
                .GroupBy(d => d.WareHouseCode)
                .Select(d => d.Key)
                .OrderBy(d => wareHouseFilters.IndexOf(d));

            WareHouseCollection = wareHouseFilters.Count == 0 ?
            wareHouseList.OrderBy(d => d).ToList() :
            wareHouseList.ToList();
            WareHouseCollection.Add("");

            SourceWarehouse = WareHouseCollection.Last();
            TargetWarehouse = WareHouseCollection.Last();

            SimpleTaskCommand = new DelegateCommand<string>(t => SimpleTaskAsync(t)).ObservesCanExecute(() => TestTaskCanExecute);
            TaskTransportCommand = new DelegateCommand<string>(t => TaskTransportAsync(t)).ObservesCanExecute(() => TestTaskCanExecute);

            AgvCallbackCommand = new DelegateCommand<string>(t => AgvCallback(t));
        }

        public void ReloadSourceWarehouse()
        {
            SourceLocationCollection = string.IsNullOrWhiteSpace(SourceWarehouse) ?
                AllLocationCollection :
                AllLocationCollection.Where(d => d.WareHouseCode == SourceWarehouse).ToList();
        }

        public void ReloadTargetWarehouse()
        {
            TargetLocationCollection = string.IsNullOrWhiteSpace(TargetWarehouse) ?
                AllLocationCollection :
                AllLocationCollection.Where(d => d.WareHouseCode == TargetWarehouse).ToList();
        }

        private void AgvCallback(string btnText)
        {
            try
            {
                var result = CustomerApi.ReportAgvARrrived(P1, P2, P3, out string content);

                var msgBoxText = $"{btnText}{result.ToChineseString()} {content}";
                MessageBox.Show(msgBoxText, Langs.Prompt, MessageBoxButton.OK, result ? MessageBoxImage.Information : MessageBoxImage.Error);
            }
            catch (Exception ex)
            {
                MessageBox.Show($"{btnText} Exception：{ex.Message}", Langs.Prompt, MessageBoxButton.OK, MessageBoxImage.Error);
            }
        }

        private async void TaskTransportAsync(string btnText)
        {
            string text = $"{btnText}？{Environment.NewLine}{SourceWarehouse}({SourceLocation?.LocationCode})->{TargetWarehouse}({TargetLocation?.LocationCode})";
            var dialogResult = MessageBox.Show(text, Langs.Prompt, MessageBoxButton.OKCancel, MessageBoxImage.Question, MessageBoxResult.Cancel);
            if (dialogResult != MessageBoxResult.OK)
            {
                return;
            }

            TestTaskCanExecute = false;
            try
            {
                RequestTaskTransport body = new RequestTaskTransport();
                body.TaskId = DateTime.Now.ToString("yyyyMMddHHmmssfffffff");
                body.StartPos = SourceLocation?.LocationCode;
                body.EndPos = TargetLocation?.LocationCode;

                var res = await Task.Run(() => PmsCustomApi.Post<object>("TaskTransport", body));
                var result = res.code == 20000;

                var msgBoxText = $"{btnText}{result.ToChineseString()} {res.message}";
                MessageBox.Show(msgBoxText, Langs.Prompt, MessageBoxButton.OK, result ? MessageBoxImage.Information : MessageBoxImage.Error);
            }
            catch (Exception ex)
            {
                MessageBox.Show($"{btnText} Exception：{ex.Message}", Langs.Prompt, MessageBoxButton.OK, MessageBoxImage.Error);
            }
            TestTaskCanExecute = true;
        }

        private async void SimpleTaskAsync(string btnText)
        {
            string text = $"{btnText}？{Environment.NewLine}{SourceWarehouse}({SourceLocation?.LocationCode})->{TargetWarehouse}({TargetLocation?.LocationCode})";
            var dialogResult = MessageBox.Show(text, Langs.Prompt, MessageBoxButton.OKCancel, MessageBoxImage.Question, MessageBoxResult.Cancel);
            if (dialogResult != MessageBoxResult.OK)
            {
                return;
            }

            TestTaskCanExecute = false;
            try
            {
                PmsTaskInfoDto data = new PmsTaskInfoDto();
                data.LocationCode = SourceLocation?.LocationCode;
                data.TargetLocationCode = TargetLocation?.LocationCode;

                var res = await Task.Run(() => PmsCustomApi.Post<object>("TaskAdd", data));
                var result = res.code == 20000;

                var msgBoxText = $"{btnText}{result.ToChineseString()} {res.message}";
                MessageBox.Show(msgBoxText, Langs.Prompt, MessageBoxButton.OK, result ? MessageBoxImage.Information : MessageBoxImage.Error);
            }
            catch (Exception ex)
            {
                MessageBox.Show($"{btnText} Exception：{ex.Message}", Langs.Prompt, MessageBoxButton.OK, MessageBoxImage.Error);
            }
            TestTaskCanExecute = true;
        }
    }
}
